@@ -1,6 +1,6 @@
 # Automatically updated by `make build`
-__version__ = "0.4.1"
+__version__ = "0.4.2"
 
 from harmony.config import Environment
-from harmony.harmony import BBox, Client, Collection, LinkType, Request
+from harmony.harmony import BBox, Client, Collection, LinkType, Request, Dimension
 from harmony.util import s3_components
