@@ -20,7 +20,7 @@ import platform
 import requests.models
 from concurrent.futures import Future, ThreadPoolExecutor
 from contextlib import contextmanager
-from datetime import datetime
+from datetime import date, datetime
 from enum import Enum
 from typing import Any, ContextManager, IO, List, Mapping, NamedTuple, Optional, Tuple, Generator
 
@@ -108,6 +108,49 @@ class BBox(NamedTuple):
         return f'BBox: West:{self.w}, South:{self.s}, East:{self.e}, North:{self.n}'
 
 
+class Dimension:
+    """An arbitrary dimension to subset against. A dimension can take a minimum value and a
+    maximum value to to subset against.
+
+    Example:
+        Requesting the data to be subset by the dimension lev with a minimum value of 10.0
+        and maximum value of 20.0::
+
+            >>> dimension = Dimension('lev', 10.0, 20.0)
+
+        Important: When specified positionally, the parameters must be given in the order:
+        dimension name, minimum value, maximum value.
+
+        Alternatively, one can explicitly set each value used named parameters:
+
+            >>> dimension = Dimension(name='lev', min=10.0, max=20.0)
+
+        Print a readable representation of the dimension::
+
+            >>> print(dimension)
+            Dimension: Name: lev, Minimum: 10.0, Maximum: 20.0
+
+    Args:
+        name: The dimension name
+        min: The minimum value for the given dimension to subset against (optional)
+        max: The maximum value for the given dimension to subset against (optional)
+
+    Returns:
+        A Dimension instance with the provided dimension subset values.
+    """
+    name: str
+    min: float
+    max: float
+
+    def __init__(self, name: str, min: float = None, max: float = None):
+        self.name = name
+        self.min = min
+        self.max = max
+
+    def __repr__(self) -> str:
+        return f'Dimension: Name: {self.name}, Minimum:{self.min}, Maximum:{self.max}'
+
+
 _shapefile_exts_to_mimes = {
     'json': 'application/geo+json',
     'geojson': 'application/geo+json',
@@ -130,6 +173,8 @@ class Request:
         temporal: Date/time constraints on the data provided as a dict mapping "start" and "stop"
           keys to corresponding start/stop datetime.datetime objects
 
+        dimensions: A list of dimensions to use for subsetting the data
+
         crs: reproject the output coverage to the given CRS.  Recognizes CRS types that can be
           inferred by gdal, including EPSG codes, Proj4 strings, and OGC URLs
           (http://www.opengis.net/def/crs/...)
@@ -155,6 +200,9 @@ class Request:
 
         concatenate: Whether to invoke a service that supports concatenation
 
+        skip_preview: Whether Harmony should skip auto-pausing and generating a preview for
+          large jobs
+
     Returns:
         A Harmony Request instance
     """
@@ -164,6 +212,7 @@ class Request:
                  *,
                  spatial: BBox = None,
                  temporal: Mapping[str, datetime] = None,
+                 dimensions: List[Dimension] = None,
                  crs: str = None,
                  format: str = None,
                  granule_id: List[str] = None,
@@ -175,12 +224,14 @@ class Request:
                  shape: Optional[Tuple[IO, str]] = None,
                  variables: List[str] = ['all'],
                  width: int = None,
-                 concatenate: bool = None):
+                 concatenate: bool = None,
+                 skip_preview: bool = None):
         """Creates a new Request instance from all specified criteria.'
         """
         self.collection = collection
         self.spatial = spatial
         self.temporal = temporal
+        self.dimensions = dimensions
         self.crs = crs
         self.format = format
         self.granule_id = granule_id
@@ -193,6 +244,7 @@ class Request:
         self.variables = variables
         self.width = width
         self.concatenate = concatenate
+        self.skip_preview = skip_preview
 
         self.variable_name_to_query_param = {
             'crs': 'outputcrs',
@@ -205,7 +257,8 @@ class Request:
             'height': 'height',
             'format': 'format',
             'max_results': 'maxResults',
-            'concatenate': 'concatenate'
+            'concatenate': 'concatenate',
+            'skip_preview': 'skipPreview',
         }
 
         self.spatial_validations = [
@@ -233,6 +286,10 @@ class Request:
              'The provided shape file is not a recognized type.  Valid file extensions: '
              + f'[{_valid_shapefile_exts}]'),
         ]
+        self.dimension_validations = [
+            (lambda dim: dim.min is None or dim.max is None or dim.min <= dim.max,
+             ('Dimension minimum value must be less than or equal to the maximum value'))
+        ]
 
     def parameter_values(self) -> List[Tuple[str, Any]]:
         """Returns tuples of each query parameter that has been set and its value."""
@@ -262,13 +319,19 @@ class Request:
         """A list of error messages, if any, for the request."""
         spatial_msgs = []
         temporal_msgs = []
+        dimension_msgs = []
         shape_msgs = self._shape_error_messages(self.shape)
         if self.spatial:
             spatial_msgs = [m for v, m in self.spatial_validations if not v(self.spatial)]
         if self.temporal:
             temporal_msgs = [m for v, m in self.temporal_validations if not v(self.temporal)]
+        if self.dimensions:
+            for dim in self.dimensions:
+                msgs = [m for v, m in self.dimension_validations if not v(dim)]
+                if msgs:
+                    dimension_msgs += msgs
 
-        return spatial_msgs + temporal_msgs + shape_msgs
+        return spatial_msgs + temporal_msgs + shape_msgs + dimension_msgs
 
 
 class LinkType(Enum):
@@ -346,6 +409,14 @@ class Client:
         """Constructs the URL for the Job that is used to get its status."""
         return f'{self.config.root_url}/jobs/{job_id}?linktype={link_type.value}'
 
+    def _pause_url(self, job_id: str, link_type: LinkType = LinkType.https) -> str:
+        """Constructs the URL for the Job that is used to pause it."""
+        return f'{self.config.root_url}/jobs/{job_id}/pause?linktype={link_type.value}'
+
+    def _resume_url(self, job_id: str, link_type: LinkType = LinkType.https) -> str:
+        """Constructs the URL for the Job that is used to resume it."""
+        return f'{self.config.root_url}/jobs/{job_id}/resume?linktype={link_type.value}'
+
     def _cloud_access_url(self) -> str:
         return f'{self.config.root_url}/cloud-access'
 
@@ -353,7 +424,10 @@ class Client:
         """Creates a dictionary of request query parameters from the given request."""
         params = {'forceAsync': 'true'}
 
-        subset = self._spatial_subset_params(request) + self._temporal_subset_params(request)
+        subset = self._spatial_subset_params(request) + \
+            self._temporal_subset_params(request) + \
+            self._dimension_subset_params(request)
+
         if len(subset) > 0:
             params['subset'] = subset
 
@@ -436,6 +510,19 @@ class Client:
         else:
             return []
 
+    def _dimension_subset_params(self, request: Request) -> list:
+        """Creates a list of dimension subset query parameters."""
+        if request.dimensions and len(request.dimensions) > 0:
+            dimensions = []
+            for dim in request.dimensions:
+                min = dim.min or '*'
+                max = dim.max or '*'
+                dim_query_param = [f'{dim.name}({min}:{max})']
+                dimensions.append(dim_query_param)
+            return dimensions
+        else:
+            return []
+
     @contextmanager
     def _files(self, request) -> ContextManager[Mapping[str, Any]]:
         """Creates a dictionary of multipart file POST parameters from the given request."""
@@ -584,26 +671,70 @@ class Client:
         response = session.get(self._status_url(job_id))
         if response.ok:
             fields = [
-                'status', 'message', 'progress', 'createdAt', 'updatedAt', 'request',
-                'numInputGranules'
+                'status', 'message', 'progress', 'createdAt', 'updatedAt', 'dataExpiration',
+                'request', 'numInputGranules'
             ]
             status_subset = {k: v for k, v in response.json().items() if k in fields}
             created_at_dt = dateutil.parser.parse(status_subset['createdAt'])
             updated_at_dt = dateutil.parser.parse(status_subset['updatedAt'])
+            data_expiration_dt = dateutil.parser.parse(status_subset['dataExpiration'])
             return {
                 'status': status_subset['status'],
-                'message': status_subset['message'],
+                'message': status_subset['message']
+                .replace(' The job may be resumed using the provided link.', ''),
                 'progress': status_subset['progress'],
                 'created_at': created_at_dt,
                 'updated_at': updated_at_dt,
                 'created_at_local': created_at_dt.replace(microsecond=0).astimezone().isoformat(),
                 'updated_at_local': updated_at_dt.replace(microsecond=0).astimezone().isoformat(),
+                'data_expiration': data_expiration_dt,
+                'data_expiration_local': data_expiration_dt.
+                replace(microsecond=0).astimezone().isoformat(),
                 'request': status_subset['request'],
                 'num_input_granules': int(status_subset['numInputGranules']),
             }
         else:
             response.raise_for_status()
 
+    def pause(self, job_id: str):
+        """Pause a job.
+
+        Args:
+            job_id: UUID string for the job you wish to pause.
+        Raises:
+            Exception: This can happen if an invalid job_id is provided or Harmony services
+              can't be reached or the job cannot be paused (usually because it is already
+              in a terminal state).
+        """
+        session = self._session()
+        response = session.get(self._pause_url(job_id))
+        if not response.ok:
+            if response.status_code == 409:
+                # 409 means we could not pause - the json will have a reason
+                message = response.json().get('description')
+                raise Exception(response.reason, message)
+            else:
+                response.raise_for_status()
+
+    def resume(self, job_id: str):
+        """Resume a job.
+
+        Args:
+            job_id: UUID string for the job you wish to resume.
+        Raises:
+            Exception: This can happen if an invalid job_id is provided or Harmony services
+              can't be reached or the job cannot be resumed (usually because it is already
+              in a terminal state).
+        """
+        session = self._session()
+        response = session.get(self._resume_url(job_id))
+        if response.status_code == 409:
+            # 409 means we could not resume - the json will have a reason
+            message = response.json().get('description')
+            raise Exception(response.reason, message)
+        else:
+            response.raise_for_status()
+
     def progress(self, job_id: str) -> Tuple[int, str, str]:
         """Retrieve a submitted job's completion status in percent.
 
@@ -655,6 +786,9 @@ class Client:
                     if status == 'canceled':
                         print('Job has been canceled.')
                         break
+                    if status == 'paused':
+                        print('\nJob has been paused. Call `resume()` to resume.', file=sys.stderr)
+                        break
                     # This gets around an issue with progressbar. If we update() with 0, the
                     # output shows up as "N/A". If we update with, e.g. 0.1, it rounds down or
                     # truncates to 0 but, importantly, actually displays that.
@@ -676,6 +810,9 @@ class Client:
                     raise ProcessingFailedException(job_id, message)
                 if status == 'canceled':
                     break
+                if status == 'paused':
+                    print('Job has been paused. Call `resume()` to resume.', file=sys.stderr)
+                    break
                 time.sleep(check_interval)
 
     def result_json(self,
